#!/usr/bin/env python

import itertools

class MarkdownGenerator:

    @staticmethod
    def generate_tablerow() -> str:
        pass


    @staticmethod
    def generate_table(table_items,add_pagebreak = False,title_template = None,first_colwidth = 28):

        import pandas as pd
        import tabulate 

        # get the dataframe
        df = pd.DataFrame(table_items)

        # use first column for 
        df.columns = df.iloc[0]
        df = df[1:]

        if title_template != None:
            print(title_template.format(df.columns[1]),'\n')

        print(df.to_markdown(tablefmt='grid', index=False, maxcolwidths=[first_colwidth,None]))
        print('\n') # always add a newline after the table

        if add_pagebreak:
            print('\\pagebreak')
        
    
    @staticmethod
    def generate_table_legacy(table_items,add_pagebreak = False,title_template = None,first_colwidth = 28):

        import textwrap

        line_length = 128
        column_ratio = float(first_colwidth) / 100

        h_len = int(line_length * column_ratio)
        d_len = line_length-h_len

        #
        #  generate title (currently the first one)
        # 
        if title_template != None:
            print(title_template.format(table_items[0][1]),'\n')

        print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))

        headline = False

        #
        # this implements a Markdown Grid-Table
        #

        # test if this affected by a third item!

        for k,v in table_items:

            # 
            if v == None:
                v = ''

            # row head
            h =  textwrap.wrap(k, h_len, break_long_words=False)
            wrapper = textwrap.TextWrapper(d_len,break_long_words=True,replace_whitespace=False)

            # split test to wrap lines into correct length
            t = [wrapper.wrap(line) for line in v.split('\n')]


            # replace empty arrays from split with a empty string
            t = list(map(lambda e: [""] if e == [] else e, t))

            # zip items of list 
            t = list(itertools.chain.from_iterable(t))
            
            # get rows
            rows = list(itertools.zip_longest(h,t,fillvalue=""))

            # expand rows
            for r in rows:
                # insider recognize this as the computational dump(b)ness feature
                if '<!-- tablebreak -->' in r[0] or '<!-- tablebreak -->' in r[1]:
                    print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))
                else: 
                    print(''.join(['|',r[0].ljust(h_len,' '),'|',r[1].ljust(d_len,' '),'|']))

            if headline:
                print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))
            else:
                print(''.join(['+',"".ljust(h_len,'='),'+',"".ljust(d_len,'='),'+']))
                headline = True


        # to control pagebreaks for pandoc
        if add_pagebreak:
            print('\n\\newpage')